%% Pre-rquirement: None
%However, Fs=10KHz, N=2^16, RX coordinate and number, Water Tank Dimension 
% must keep same through step0-3.
%% Function:
%Generate Green Function Dictionary for Water Tank Grid 20x20x20 for three
%different models.

%% ---------------------------Add all functions path-------------------------
path1 = 'C:\Users\yx101\Desktop'; %path that includes Data repository
%path that includes Code for all functions
path2 = '\Data repository\3-programs and algorithms\Code\Code for all functions';
addpath([path1,path2,'\alg_Green_Function']);
%% ------------------------------Main---------------------------------
Fs = 10^4;% Fs=10KHz, RX signals sample frequency, same as step1 Fsd
N = 2^16;% number of FFT, fixed, same as positiveFFT_fixN
a=0:1:(ceil(N/2)-1);
SInput_freqR=Fs/N.*a;
f = SInput_freqR;
%------------------coordinate of scattered hydorphones---------------------
CH_Cdnate =[49,30,121;    26,97.5,74;    52,165,162;...
            131,158,85.5; 191,111,136.5; 164.5,44,135;...
            94,125,83.5].*0.01; %unit:m, 7 RX sensors
numSensorScatter = 7;%Number of sensors = 7
% -------------------------Water Tank Dimension----------------------------
 L = [2.1336  2.1336  1.6788]; 
 h = 0.0254;                % wall thickness [m]
 number_of_modes = 15;      % number of modes
%---------Source coordinate grid,[x,y,z] have 20 points respectively-------
% Totally, 20*20*20=8000 possibble coordinates choices
Source_Coord(1,1:20) = L(1)/40:L(1)/20:(L(1)-L(1)/40);
Source_Coord(2,1:20) = L(2)/40:L(2)/20:(L(2)-L(2)/40);
Source_Coord(3,1:20) = L(3)/40:L(3)/20:(L(3)-L(3)/40);
%------------Calculate corresponding frequency index-----------------------
w1 = 600;%unit:Hz
w2 = 2e3;%unit:Hz
%------------Calculate corresponding frequency index-----------------------
frq_interval = SInput_freqR(2);
fre_index1 = round(w1/frq_interval+1);
fre_index2 = round(w2/frq_interval+1);
%--------------------Water Tank Grids (20x20x20)---------------------------
GridD = 20;
DCL_Dict = cell(numSensorScatter,GridD,GridD,GridD);
MIX_Dict = cell(numSensorScatter,GridD,GridD,GridD);
FRF_Dict = cell(numSensorScatter,GridD,GridD,GridD);
%% -------------------------------Description-------------------------
%Since the Green Function Library under Grids (20x20x20) is big, if
%generating the library at one time, the matlab will report error: out of 
%memory, so in program, we seperate the library to four parts (m1,m2,m345,m67) for each 
%model.(so below 4 parts can be divided to four .m file with same above Main part,
% so that the 4 programs can be run at different time or run at different 
%computer and solve out of memory problem.)
%% -------Part1: Green Function Library: m=1 for each model-----------
for m=1
    rm = CH_Cdnate(m,:);     % Scatter hydrophone position [x y z] [m]
    for i=1:GridD
        for j=1:GridD
            for k=1:GridD
                r0 = [Source_Coord(1,i),Source_Coord(2,j),Source_Coord(3,k)];% source
                %M1:Call real Dirchlet function
                DCLm = M1_Tank_Dirchlet_funcion(L,r0,rm,h,number_of_modes,2*pi*f);
                DCL_Dict{m,i,j,k} = DCLm;
                %Proposed: Call Mix (Dirchlet and FRF) function
                MIXm = Proposed_Tank_MIX_function(L,r0,rm,h,number_of_modes,2*pi*f);
                MIX_Dict{m,i,j,k} = MIXm;
                %M2:Call FRF function
                FRFm = M2_Tank_FRF_function(L,r0,rm,h,number_of_modes,2*pi*f);
                FRF_Dict{m,i,j,k} = FRFm;
            end % for k
        end % for j
    end % for i
       
end % for m
DLC_Dict_G20m1 = DCL_Dict;
FRF_Dict_G20m1 = FRF_Dict;
MIX_Dict_G20m1 = MIX_Dict;
%Green Function Dictionary 20x20x20, m=1 part
save('DLC_Dict_G20m1.mat','DLC_Dict_G20m1');
save('FRF_Dict_G20m1.mat','FRF_Dict_G20m1');
save('MIX_Dict_G20m1.mat','MIX_Dict_G20m1');

%% ---Part2: Green Function Library: m=2 for each model------------
for m=2
    rm = CH_Cdnate(m,:);     % Scatter hydrophone position [x y z] [m]
    for i=1:GridD
        for j=1:GridD
            for k=1:GridD
                r0 = [Source_Coord(1,i),Source_Coord(2,j),Source_Coord(3,k)];% source
                %M1:Call real Dirchlet function
                DCLm = M1_Tank_Dirchlet_funcion(L,r0,rm,h,number_of_modes,2*pi*f);
                DCL_Dict{m,i,j,k} = DCLm;
                %Proposed: Call Mix (Dirchlet and FRF) function
                MIXm = Proposed_Tank_MIX_function(L,r0,rm,h,number_of_modes,2*pi*f);
                MIX_Dict{m,i,j,k} = MIXm;
                %M2:Call FRF function
                FRFm = M2_Tank_FRF_function(L,r0,rm,h,number_of_modes,2*pi*f);
                FRF_Dict{m,i,j,k} = FRFm;
            end % for k
        end % for j
    end % for i
       
end % for m
%Green Function Dictionary 20x20x20,m=2 part
DLC_Dict_G20m2 = DCL_Dict;
FRF_Dict_G20m2 = FRF_Dict;
MIX_Dict_G20m2 = MIX_Dict;
%Green Function Dictionary 20x20x20, m=2 part
save('DLC_Dict_G20m2.mat','DLC_Dict_G20m2');
save('FRF_Dict_G20m2.mat','FRF_Dict_G20m2');
save('MIX_Dict_G20m2.mat','MIX_Dict_G20m2');

%% --part3: Green Function Library: m=345 for each model------------
for m=3:5
    rm = CH_Cdnate(m,:);     % Scatter hydrophone position [x y z] [m]
    for i=1:GridD
        for j=1:GridD
            for k=1:GridD
                r0 = [Source_Coord(1,i),Source_Coord(2,j),Source_Coord(3,k)];% source
                %M1:Call real Dirchlet function
                DCLm = M1_Tank_Dirchlet_funcion(L,r0,rm,h,number_of_modes,2*pi*f);
                DCL_Dict{m,i,j,k} = DCLm;
                %Proposed: Call Mix (Dirchlet and FRF) function
                MIXm = Proposed_Tank_MIX_function(L,r0,rm,h,number_of_modes,2*pi*f);
                MIX_Dict{m,i,j,k} = MIXm;
                %M2:Call FRF function
                FRFm = M2_Tank_FRF_function(L,r0,rm,h,number_of_modes,2*pi*f);
                FRF_Dict{m,i,j,k} = FRFm;
            end % for k
        end % for j
    end % for i
end % for m
%Green Function Dictionary 20x20x20, m=345 part
DLC_Dict_G20m345 = DCL_Dict;
FRF_Dict_G20m345 = FRF_Dict;
MIX_Dict_G20m345 = MIX_Dict;
%Green Function Dictionary 20x20x20, m=1 part
save('DLC_Dict_G20m345.mat','DLC_Dict_G20m345');
save('FRF_Dict_G20m345.mat','FRF_Dict_G20m345');
save('MIX_Dict_G20m345.mat','MIX_Dict_G20m345');

%% --part4: Green Function Library: m=67 for each model------------
for m=6:7
    rm = CH_Cdnate(m,:);     % Scatter hydrophone position [x y z] [m]
    for i=1:GridD
        for j=1:GridD
            for k=1:GridD
                r0 = [Source_Coord(1,i),Source_Coord(2,j),Source_Coord(3,k)];% source
                %M1:Call real Dirchlet function
                DCLm = M1_Tank_Dirchlet_funcion(L,r0,rm,h,number_of_modes,2*pi*f);
                DCL_Dict{m,i,j,k} = DCLm;
                %Proposed: Call Mix (Dirchlet and FRF) function
                MIXm = Proposed_Tank_MIX_function(L,r0,rm,h,number_of_modes,2*pi*f);
                MIX_Dict{m,i,j,k} = MIXm;
                %M2:Call FRF function
                FRFm = M2_Tank_FRF_function(L,r0,rm,h,number_of_modes,2*pi*f);
                FRF_Dict{m,i,j,k} = FRFm;
            end % for k
        end % for j
    end % for i
end % for m
%Green Function Dictionary 20x20x20, m=67 part
DLC_Dict_G20 = DCL_Dict;
FRF_Dict_G20 = FRF_Dict;
MIX_Dict_G20 = MIX_Dict;
%Green Function Dictionary 20x20x20, m=67 part
save('DLC_Dict_G20m67.mat','DLC_Dict_G20');
save('FRF_Dict_G20m67.mat','FRF_Dict_G20');
save('MIX_Dict_G20m67.mat','MIX_Dict_G20');
