%% Pre-Requirements: 
% 1. Step0_Green_Function_Library-Grid20.m
% 2. Step2_Process_RX_signals.m

%% Function: SpectraRatio-based localization with different models
% under water tank grids 20x20x20

%Input: 
%   Step2: Frequency Domain Signal and Frequency Rangle
%   Step0: Green Function dictionary under grids 20x20x20: 
%   DLC_Dict_G20m1.mat;DLC_Dict_G20m2.mat;DLC_Dict_G20m345.mat;DCL_Dict_G20m67.mat
%   Water tank dimension, RX,TX coordinates and number.
%Output:
%   Estimation result: EST_M2
%   RMSE result:,rmse_M2
%% ----------------------Important Parameter--------------------------
% -----------------Load Green Function Dictionary--------------------------
%Generated by Step0_Green_Function_Library-Grid20.m
load('FRF_Dict_G20m1.mat');%FRF_Dict_G20m1
load('FRF_Dict_G20m2.mat');%FRF_Dict_G20m2
load('FRF_Dict_G20m345.mat');%FRF_Dict_G20m345
load('FRF_Dict_G20m67.mat');%FRF_Dict_G20
% -------------------True Bubble Sources Coordinate------------------------
s4=[1.46 1.12 1.48];
s5=[0.71 1.03 1.29];
s6 = [1.02 1.185 1.47];  
s7 = [1.465 1.135 1.29];
SN='Source S7';
source = s7;
%------------------coordinate of scattered hydorphones---------------------
CH_Cdnate =[49,30,121;    26,97.5,74;    52,165,162;...
            131,158,85.5; 191,111,136.5; 164.5,44,135;...
            94,125,83.5].*0.01; %unit:m, 7 RX sensors
% -------------------------Water Tank Dimension----------------------------
L = [2.1336  2.1336  1.6788]; %True Tank Dimensions[Lx Ly Lz][m]
h = 0.0254; %wall thickness [m]
number_of_modes = 15;  %number of modes
% -----------------RX signals frequency range, get by step1----------------
f = SInput_freqR;
% ---------------------Number of scattered hydrophones---------------------
numSensorScatter = 7;
%% ---SpectraRatio-based Localization by Scattered RXs,three models---
%----------Choose intrested frequency range for Green Function-------------
w1 = 600;%unit:Hz
w2 = 2e3;%unit:Hz
%------------Calculate corresponding frequency index-----------------------
frq_interval = SInput_freqR(2);
fre_index1 = round(w1/frq_interval+1);
fre_index2 = round(w2/frq_interval+1);
%---------Source coordinate grid,[x,y,z] have 20 points respectively-------
%Totally, 20*20*20=8000 possibble coordinates choices
Source_Coord(1,1:20) = L(1)/40:L(1)/20:(L(1)-L(1)/40);
Source_Coord(2,1:20) = L(2)/40:L(2)/20:(L(2)-L(2)/40);
Source_Coord(3,1:20) = L(3)/40:L(3)/20:(L(3)-L(3)/40);
%search all possible RX sensor pairs=7 choose 2=7*6/2=21 cases
%1 possible source coordinate=21 cases RX sensor pairs=1 DF
GridD = 20;
for i=1:GridD
    for j=1:GridD
        for k=1:GridD
            %Choose possibble Source Coordinate
            %r0 = [Source_Coord(1,i),Source_Coord(2,j),Source_Coord(3,k)];% source
            %Initial Detection Factor
             FRFDF(i,j,k) = 1;
%             MIXDF(i,j,k) = 1;
%            DCLDF(i,j,k) = 1;
            for m=1:(numSensorScatter-1)
                if m==1
                    FRF_Dictm = FRF_Dict_G20m1;
                elseif m==2
                    FRF_Dictm = FRF_Dict_G20m2;
                elseif m==3 || m==4 ||m==5
                    FRF_Dictm = FRF_Dict_G20m345;
                else
                    FRF_Dictm = FRF_Dict_G20;
                end
                    
                for n=(m+1):numSensorScatter
                    if n==1
                        FRF_Dictn = FRF_Dict_G20m1;
                    elseif n==2
                        FRF_Dictn = FRF_Dict_G20m2;
                    elseif n==3 || n==4 || n==5
                        FRF_Dictn = FRF_Dict_G20m345;
                    else
                        FRF_Dictn = FRF_Dict_G20;
                    end
                
                    RPower = (CHFSInput_FFT{n}./CHFSInput_FFT{m}).';
                    
                    % FRF model
                    %search all source possible coordinate
                    FRFm = FRF_Dictm{m,i,j,k};
                    FRFn = FRF_Dictn{n,i,j,k};
                    FRFGreen = FRFm./FRFn;
                    FRFSpectraR = FRFGreen.*RPower;
                    FRFSpectraR_dB = (20*log10(abs(FRFSpectraR))).^2;
                    FRFRMS_S = sqrt(1/(w2-w1)*trapz(SInput_freqR(...
                        fre_index1:fre_index2),FRFSpectraR_dB(fre_index1:fre_index2)));
                    FRFDF(i,j,k) = FRFDF(i,j,k)*FRFRMS_S;
                end
            end
             FRFDF(i,j,k) = 1./FRFDF(i,j,k);
        end
    end
end

%%
% FRF localization result
[~,zFRF,~] = find(max(max(FRFDF))==max(max(max(FRFDF))));
DFz = FRFDF(:,:,zFRF);
[xFRF,yFRF] = find(DFz==max(max(max(FRFDF))));
%Estimated source coordinate 
EST2_FRF = [Source_Coord(1,xFRF),Source_Coord(2,yFRF),Source_Coord(3,zFRF)];
%Check maximum value
DF_max_FRF = max(max(max(FRFDF)));
DF_max2_FRF = FRFDF(xFRF,yFRF,zFRF);
%Root mean square value
rmse_FRF = rms(source-EST2_FRF,2);
coord_FRF =[xFRF,yFRF,zFRF];
EST_M2 = EST2_FRF;
rmse_M2 = rmse_FRF;

filename = [Hydrophone_data,'_M2_result_',num2str(bg),'to',num2str(ed),'_G20_RN(1-7)'];
save(filename);