%% Pre-Requirements: 
% 1. Step0_Green_Function_Library-Grid20.m
% 2. Step2_Process_RX_signals.m

%% Function: SpectraRatio-based localization with different models
% under water tank grids 20x20x20

%Input: 
%   Step2: Frequency Domain Signal and Frequency Rangle
%   Step0: Green Function dictionary under grids 20x20x20: 
%   DLC_Dict_G20m1.mat;DLC_Dict_G20m2.mat;DLC_Dict_G20m345.mat;DCL_Dict_G20m67.mat
%   Water tank dimension, RX,TX coordinates and number.
%Output:
%   Estimation result: EST_Proposed
%   RMSE result:,rmse_Proposed
%% ----------------------Important Parameter--------------------------
% -----------------Load Green Function Dictionary--------------------------
%Generated by Step0_Green_Function_Library-Grid20.m
load('MIX_Dict_G20m1.mat');% MIX_Dict_G20m1
load('MIX_Dict_G20m2.mat');%MIX_Dict_G20m2
load('MIX_Dict_G20m345.mat');%MIX_Dict_G20m345
load('MIX_Dict_G20m67.mat');%MIX_Dict_G20
% -------------------True Bubble Sources Coordinate------------------------
s4=[1.46 1.12 1.48];
s5=[0.71 1.03 1.29];
s6 = [1.02 1.185 1.47];  
s7 = [1.465 1.135 1.29];
SN='Source S7';
source = s7;
%------------------coordinate of scattered hydorphones---------------------
CH_Cdnate =[49,30,121;    26,97.5,74;    52,165,162;...
            131,158,85.5; 191,111,136.5; 164.5,44,135;...
            94,125,83.5].*0.01; %unit:m, 7 RX sensors
% -------------------------Water Tank Dimension----------------------------
L = [2.1336  2.1336  1.6788]; %True Tank Dimensions[Lx Ly Lz][m]
h = 0.0254; %wall thickness [m]
number_of_modes = 15;  %number of modes
% -----------------RX signals frequency range, get by step1----------------
f = SInput_freqR;
% ---------------------Number of scattered hydrophones---------------------
numSensorScatter = 7;
%% ---SpectraRatio-based Localization by Scattered RXs,three models---
%----------Choose intrested frequency range for Green Function-------------
w1 = 600;%unit:Hz
w2 = 2e3;%unit:Hz
%------------Calculate corresponding frequency index-----------------------
frq_interval = SInput_freqR(2);
fre_index1 = round(w1/frq_interval+1);
fre_index2 = round(w2/frq_interval+1);
%---------Source coordinate grid,[x,y,z] have 20 points respectively-------
%Totally, 20*20*20=8000 possibble coordinates choices
Source_Coord(1,1:20) = L(1)/40:L(1)/20:(L(1)-L(1)/40);
Source_Coord(2,1:20) = L(2)/40:L(2)/20:(L(2)-L(2)/40);
Source_Coord(3,1:20) = L(3)/40:L(3)/20:(L(3)-L(3)/40);
%search all possible RX sensor pairs=7 choose 2=7*6/2=21 cases
%1 possible source coordinate=21 cases RX sensor pairs=1 DF
GridD = 20;
for i=1:GridD
    for j=1:GridD
        for k=1:GridD
            %Initial Detection Factor
            MIXDF(i,j,k) = 1;
            for m=1:(numSensorScatter-1)
                if m==1
                    MIX_Dictm = MIX_Dict_G20m1;
                elseif m==2
                    MIX_Dictm = MIX_Dict_G20m2;
                elseif m==3 || m==4 ||m==5
                    MIX_Dictm = MIX_Dict_G20m345;
                else
                    MIX_Dictm = MIX_Dict_G20;
                end
                    
                for n=(m+1):numSensorScatter
                    if n==1
                        MIX_Dictn = MIX_Dict_G20m1;
                    elseif n==2
                        MIX_Dictn = MIX_Dict_G20m2;
                    elseif n==3 || n==4 || n==5
                        MIX_Dictn = MIX_Dict_G20m345;
                    else
                        MIX_Dictn = MIX_Dict_G20;
                    end
                
                    RPower = (CHFSInput_FFT{n}./CHFSInput_FFT{m}).';
   
                    % MIX model
                    %search all source possible coordinate
                    MIXm = MIX_Dictm{m,i,j,k};
                    MIXn = MIX_Dictn{n,i,j,k};
                    MIXGreen = MIXm./MIXn;
                    MIXSpectraR = MIXGreen.*RPower;
                    MIXSpectraR_dB = (20*log10(abs(MIXSpectraR))).^2;
                    MIXRMS_S = sqrt(1/(w2-w1)*trapz(SInput_freqR(...
                        fre_index1:fre_index2),MIXSpectraR_dB(fre_index1:fre_index2)));
                    MIXDF(i,j,k) = MIXDF(i,j,k)*MIXRMS_S;
                end
            end
             MIXDF(i,j,k) = 1./MIXDF(i,j,k);
        end
    end
end
%%
% MIX localization result
[~,zMIX,~] = find(max(max(MIXDF))==max(max(max(MIXDF))));
DFz = MIXDF(:,:,zMIX);
[xMIX,yMIX] = find(DFz==max(max(max(MIXDF))));
%Estimated source coordinate 
EST2_MIX = [Source_Coord(1,xMIX),Source_Coord(2,yMIX),Source_Coord(3,zMIX)];
%Check maximum value
DF_max_MIX = max(max(max(MIXDF)));
DF_max2_MIX = MIXDF(xMIX,yMIX,zMIX);
%Root mean square value
rmse_MIX = rms(source-EST2_MIX,2);
coord_MIX =[xMIX,yMIX,zMIX];
EST_Proposed =EST2_MIX;
rmse_Proposed = rmse_MIX;

filename = [Hydrophone_data,'_Proposed_result_',num2str(bg),'to',num2str(ed),'_G20_RN(1-7)'];
save(filename);