%% Pre-Requirements: 
% 1. Step0_Green_Function_Library_Grid10.m
% 2. Step2_Process_RX_signals.m

%% Function: SpectraRatio-based localization with different models
% under water tank grids 10x10x10

%Input: 
%   Step2: Frequency Domain Signal and Frequency Rangle
%   Step0: Green Function dictionary under grids 10x10x10: Dic_DCL.mat;Dic_FRF.mat;Dic_MIX.mat
%   Water tank dimension, RX,TX coordinates and number.
%Output:
%   Estimation result: EST_M1,EST_M2,EST_Proposed
%   RMSE result:,rmse_M1,rmse_M2,rmse_Proposed 
%% ----------------------Important Parameter--------------------------
% -----------------Load Green Function Dictionary--------------------------
%Generated by Step0_Green_Function_Library_Grid10.m
load('Dic_DCL.mat');
load('Dic_FRF.mat');
load('Dic_MIX.mat');
% -------------------True Bubble Sources Coordinate------------------------
s4=[1.46 1.12 1.48];
s5=[0.71 1.03 1.29];
s6 = [1.02 1.185 1.47];  
s7 = [1.465 1.135 1.29];
SN='Source S7';
source = s7;
%------------------coordinate of scattered hydorphones---------------------
CH_Cdnate =[49,30,121;    26,97.5,74;    52,165,162;...
            131,158,85.5; 191,111,136.5; 164.5,44,135;...
            94,125,83.5].*0.01; %unit:m, 7 RX sensors
% -------------------------Water Tank Dimension----------------------------
L = [2.1336  2.1336  1.6788]; %True Tank Dimensions[Lx Ly Lz][m]
h = 0.0254; %wall thickness [m]
number_of_modes = 15;  %number of modes
% -----------------RX signals frequency range, get by step1----------------
f = SInput_freqR;
% ---------------------Number of scattered hydrophones---------------------
numSensorScatter = 7;
%% ---SpectraRatio-based Localization by Scattered RXs,three models---
%----------Choose intrested frequency range for Green Function-------------
w1 = 600;%unit:Hz
w2 = 2e3;%unit:Hz
%------------Calculate corresponding frequency index-----------------------
frq_interval = SInput_freqR(2);
fre_index1 = round(w1/frq_interval+1);
fre_index2 = round(w2/frq_interval+1);
%---------Source coordinate grid,[x,y,z] have 10 points respectively-------
%Totally, 10*10*10=1000 possibble coordinates choices
Source_Coord(1,1:10) = L(1)/20:L(1)/10:(L(1)-L(1)/20);
Source_Coord(2,1:10) = L(2)/20:L(2)/10:(L(2)-L(2)/20);
Source_Coord(3,1:10) = L(3)/20:L(3)/10:(L(3)-L(3)/20);
%search all possible RX sensor pairs=7 choose 2=7*6/2=21 cases
%1 possible source coordinate=21 cases RX sensor pairs=1 DF
GridD = 10;
for i=1:GridD
    for j=1:GridD
        for k=1:GridD
            %Initial Detection Factor
            FRFDF(i,j,k) = 1;
            MIXDF(i,j,k) = 1;
            DCLDF(i,j,k) = 1;
            for m=1:(numSensorScatter-1)
                for n=(m+1):numSensorScatter
                    RPower = (CHFSInput_FFT{n}./CHFSInput_FFT{m}).';
                    %M2 model
                    %search all source possible coordinate
                    FRFm = FRF_Dict{m,i,j,k};
                    FRFn = FRF_Dict{n,i,j,k};
                    FRFGreen = FRFm./FRFn;
                    FRFSpectraR = FRFGreen.*RPower;
                    FRFSpectraR_dB = (20*log10(abs(FRFSpectraR))).^2;
                    FRFRMS_S = sqrt(1/(w2-w1)*trapz(SInput_freqR(...
                        fre_index1:fre_index2),FRFSpectraR_dB(fre_index1:fre_index2)));
                    FRFDF(i,j,k) = FRFDF(i,j,k)*FRFRMS_S;
                    
                    %Proposed model
                    %search all source possible coordinate
                    MIXm = MIX_Dict{m,i,j,k};
                    MIXn = MIX_Dict{n,i,j,k};
                    MIXGreen = MIXm./MIXn;
                    MIXSpectraR = MIXGreen.*RPower;
                    MIXSpectraR_dB = (20*log10(abs(MIXSpectraR))).^2;
                    MIXRMS_S = sqrt(1/(w2-w1)*trapz(SInput_freqR(...
                        fre_index1:fre_index2),MIXSpectraR_dB(fre_index1:fre_index2)));
                    MIXDF(i,j,k) = MIXDF(i,j,k)*MIXRMS_S;
                    
                    %M1 model
                    %search all source possible coordinate
                    DCLm = DCL_Dict{m,i,j,k};
                    DCLn = DCL_Dict{n,i,j,k};
                    DCLGreen = DCLm./DCLn;
                    DCLSpectraR = DCLGreen.*RPower;
                    DCLSpectraR_dB = (20*log10(abs(DCLSpectraR))).^2;
                    DCLRMS_S = sqrt(1/(w2-w1)*trapz(SInput_freqR(...
                        fre_index1:fre_index2),DCLSpectraR_dB(fre_index1:fre_index2)));
                    DCLDF(i,j,k) = DCLDF(i,j,k)*DCLRMS_S;
                end
            end
            FRFDF(i,j,k) = 1./FRFDF(i,j,k);
            MIXDF(i,j,k) = 1./MIXDF(i,j,k);
            DCLDF(i,j,k) = 1./DCLDF(i,j,k);
        end
    end
end
%% -------------------------Plot Proposed DF-------------------------
a=zeros(1000,4);
b=zeros(1000,4);
for i=1:GridD
    for j=1:GridD
        for k=1:GridD
            a((i-1)*100+(j-1)*10+(k-1)+1,:)=[i,j,k,MIXDF(i,j,k)];
            b((i-1)*100+(j-1)*10+(k-1)+1,:)=[Source_Coord(1,i),Source_Coord(2,j),Source_Coord(3,k),MIXDF(i,j,k)];
        end
    end
end
figure
scatter3(b(:,1), b(:,2), b(:,3), 60, b(:,4), 'filled')
view(-34, 14);
title('DF');
xlabel('x [m]');
ylabel('y [m]');
zlabel('z [m]');
colorbar('Location', 'EastOutside');
mycolormap = customcolormap([0 0.5 1], {'#FFFF00','#00BFFF','#0000FF'});
colorbar;
colormap(mycolormap);

a=zeros(100,3);
b=zeros(100,3);
for i=1:GridD
    for j=1:GridD
        for k=7
            a((i-1)*100+(j-1)*10+(k-1)+1,:)=[i,j,MIXDF(i,j,k)];
            b((i-1)*100+(j-1)*10+(k-1)+1,:)=[Source_Coord(1,i),Source_Coord(2,j),MIXDF(i,j,k)];
        end
    end
end
figure
scatter(b(:,1), b(:,2), 200, b(:,3), 'filled')
view(-34, 14);
xlabel('x [m]');
ylabel('y [m]');
title('z [m] (grid = 7, z=1.091)');
xticks([0 0.11 0.32 0.53 0.75 0.96 1.17 1.39 1.60 1.81 2.03 2.13]);
xticklabels({'0','0.11','0.32','0.53','0.75','0.96','1.17','1.39','1.60','1.81','2.03','2.13'});
yticks([0 0.11 0.32 0.53 0.75 0.96 1.17 1.39 1.60 1.81 2.03 2.13]);
yticklabels({'0','0.11','0.32','0.53','0.75','0.96','1.17','1.39','1.60','1.81','2.03','2.13'});
xlim([0 2.13]);
ylim([0 2.13]);
grid on;
colorbar('Location', 'EastOutside');
mycolormap = customcolormap([0 0.5 1], {'#FFFF00','#00BFFF','#0000FF'});
colorbar;
colormap(mycolormap);
%{'#DC143C','#00BFFF','#0000FF'}{'#FFFF00','#00BFFF','#0000FF'};{{'#0000FF','#00BFFF','#FFFF00'};'#000000','#ff8033','#ffff99'}[0 0 0; 1 0 0; 1 1 1];{'#00008B','#00BFFF','#ffff99'};{'#00008B','#00FF00','#FFFF00'}
%% -----------------Calculate Estimate Coordinate and RMSE------------
% FRF localization result
[~,zFRF,~] = find(max(max(FRFDF))==max(max(max(FRFDF))));
DFz = FRFDF(:,:,zFRF);
[xFRF,yFRF] = find(DFz==max(max(max(FRFDF))));
%Estimated source coordinate 
EST2_FRF = [Source_Coord(1,xFRF),Source_Coord(2,yFRF),Source_Coord(3,zFRF)];
%Check maximum value
DF_max_FRF = max(max(max(FRFDF)));
DF_max2_FRF = FRFDF(xFRF,yFRF,zFRF);
%Root mean square value
rmse_FRF = rms(source-EST2_FRF,2);
coord_FRF =[xFRF,yFRF,zFRF];
EST_M2 = EST2_FRF;
rmse_M2 = rmse_FRF;

% MIX localization result
[~,zMIX,~] = find(max(max(MIXDF))==max(max(max(MIXDF))));
DFz = MIXDF(:,:,zMIX);
[xMIX,yMIX] = find(DFz==max(max(max(MIXDF))));
%Estimated source coordinate 
EST2_MIX = [Source_Coord(1,xMIX),Source_Coord(2,yMIX),Source_Coord(3,zMIX)];
%Check maximum value
DF_max_MIX = max(max(max(MIXDF)));
DF_max2_MIX = MIXDF(xMIX,yMIX,zMIX);
%Root mean square value
rmse_MIX = rms(source-EST2_MIX,2);
coord_MIX =[xMIX,yMIX,zMIX];
EST_Proposed = EST2_MIX;
rmse_Proposed = rmse_MIX;


% DCL localization result
[~,zDCL,~] = find(max(max(DCLDF))==max(max(max(DCLDF))));
DFz = DCLDF(:,:,zDCL);
[xDCL,yDCL] = find(DFz==max(max(max(DCLDF))));
%Estimated source coordinate 
EST2_DCL = [Source_Coord(1,xDCL),Source_Coord(2,yDCL),Source_Coord(3,zDCL)];
%Check maximum value
DF_max_DCL = max(max(max(DCLDF)));
DF_max2_DCL = DCLDF(xDCL,yDCL,zDCL);
%Root mean square value
rmse_DCL = rms(source-EST2_DCL,2);
coord_DCL =[xDCL,yDCL,zDCL];
EST_M1 = EST2_DCL;
rmse_M1 = rmse_DCL;

filename = [Hydrophone_data,'result_',num2str(bg),'to',num2str(ed),'RN7(1-7)'];
save(filename);