function FRF = M1_Tank_Dirchlet_funcion(L,r0,r,h,number_of_modes,omega)

%% model of water tank
%
% Calculates the Green Function (Frequency Response Function) inside
% a water tank with given dimensions L = [Lx Ly Lz] between the source
% position r0 = [x0 y0 z0] and receiver position r = [x y z].
%
% FRF = small_tank_FRF_function(L,r0,r,h,number_of_modes,omega)
%
% inputs:    L               ... water tank dimensions [Lx, Ly, Lz]
%            r0              ... source position [x0 y0 z0]
%            r               ... receiver position [x y z]
%            h               ... wall thickness
%            number_of_modes ... number of modes
%            omega           ... angular frequency (2*pi*f)
%
% output:    FRF             ... Frequency Repsonse Function
%%
% wall properties
c_w = 2690; rho_w = 1180; Zw = c_w * rho_w;
% liquid properties
%c_l = 1488; rho_l = 998; Zl = c_l * rho_l;
c_l = 1500; rho_l = 1000; Zl = c_l * rho_l;
% air properties
c_a = 340;  rho_a = 1.255; Za = c_a * rho_a;

% wave number
kl = omega./c_l; % liquid

%%
psum = 0;
for mx=1:number_of_modes
    display(['mode n.' num2str(mx)]);
    for my=1:number_of_modes
        for mz=1:number_of_modes
            % Dirichlet (0th order) modal wave-number components (in
            % liquid)[N-2]
            kmx = mx*pi/L(1);
            kmy = my*pi/L(2);
            kmz = mz*pi/L(3);
            % (0th order) modulus of the  wave-number (in liquid)[N-2]
            km = sqrt(kmx.^2 + kmy.^2 + kmz.^2);
            km2 = kmx.^2 + kmy.^2 + kmz.^2;
            %Change Z-domain
            Psi0 = sin(kmx*r0(1)) .* sin(kmy*r0(2)) .* sin(kmz*r0(3));
            Psir  = sin(kmx*r(1)) .* sin(kmy*r(2)) .* sin(kmz*r(3));
            psum = psum + Psir.*Psi0 ./ (km2-kl.^2);
        end
    end
end

FRF = sqrt(2^3./prod(L)) * psum;